package com.kmalbz.database.service

import com.kmalbz.database.DatabaseFactory.dbQuery
import com.kmalbz.database.model.InputObject
import com.kmalbz.database.dao.InputObjects
import com.kmalbz.api.model.ApiObject
import org.jetbrains.exposed.sql.ResultRow
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction
import java.time.LocalDate


class InputObjectService : IInputObjectService {

    override fun addOne(apiObject: ApiObject) {
        transaction {
            InputObject.new {
                tag = apiObject.tag
                probability = apiObject.probability
            }
        }
    }

    override suspend fun getAllResultObjects(): List<ApiObject> = dbQuery {
        InputObjects.selectAll().map { toResultObject(it) }
    }

    override suspend fun getResultObjectbyTag(tag: String): ApiObject? = dbQuery {
        InputObjects.select {
            (InputObjects.tag eq tag)
        }.mapNotNull { toResultObject(it) }
            .singleOrNull()
    }

    override suspend fun getResultObjectbyDate(date: LocalDate): List<ApiObject>? = dbQuery {
        InputObjects.select {
            (InputObjects.date eq date)
        }.mapNotNull { toResultObject(it) }
    }

    override suspend fun getResultObjectbeforeDate(date: LocalDate): List<ApiObject>? = dbQuery {
        InputObjects.select {
            (InputObjects.date less  date)
        }.mapNotNull { toResultObject(it) }
    }

    override suspend fun getResultObjectafterDate(date: LocalDate): List<ApiObject>? = dbQuery {
        InputObjects.select {
            (InputObjects.date greater  date)
        }.mapNotNull { toResultObject(it) }
    }

    override suspend fun getResultObjecGreaterthanProbability(probability: Double): List<ApiObject>? = dbQuery {
        InputObjects.select {
            (InputObjects.probability greater  probability)
        }.mapNotNull { toResultObject(it) }
    }

    override suspend fun getResultObjecLessthanProbability(probability: Double): List<ApiObject>? = dbQuery {
        InputObjects.select {
            (InputObjects.probability less  probability)
        }.mapNotNull { toResultObject(it) }
    }

    override suspend fun getResultObjecEqualsProbability(probability: Double): List<ApiObject>? = dbQuery {
        InputObjects.select {
            (InputObjects.probability eq  probability)
        }.mapNotNull { toResultObject(it) }
    }

    private fun toResultObject(row: ResultRow): ApiObject =
        ApiObject(
            tag = row[InputObjects.tag],
            probability = row[InputObjects.probability]
        )
}